<style>
    .menu-title {
        font-weight: bold;
        margin-left: 10px;
    }
</style>
{% if current_user.profile_image_url %}
    <img class="shaft-profile-avatar-40" style="margin-bottom:-20px; margin-top:-20px" src="{{ current_user.profile_image_url }}">
{% endif %}
<span class="menu-title">{{ current_user.name }}</span>
