img.responsive {
  display: inline-block;
  height: auto;
  max-width: 100%; }

.responsive-image {
  border-radius: 4px; }
  .responsive-image img {
    display: inline-block;
    height: auto;
    max-width: 100%; }

.image-circle {
  border-radius: 50%; }

img.shaft-profile-avatar {
  position: relative;
  -webkit-border-radius: 50%;
  -moz-border-radius: 50%;
  border-radius: 50%;
  width: 100px;
  height: 100px; }
  img.shaft-profile-avatar-30 {
    position: relative;
    -webkit-border-radius: 50%;
    -moz-border-radius: 50%;
    border-radius: 50%;
    width: 30px;
    height: 30px; }
  img.shaft-profile-avatar-40 {
    position: relative;
    -webkit-border-radius: 50%;
    -moz-border-radius: 50%;
    border-radius: 50%;
    width: 40px;
    height: 40px; }
  img.shaft-profile-avatar-50 {
    position: relative;
    -webkit-border-radius: 50%;
    -moz-border-radius: 50%;
    border-radius: 50%;
    width: 50px;
    height: 50px; }
  img.shaft-profile-avatar-100 {
    position: relative;
    -webkit-border-radius: 50%;
    -moz-border-radius: 50%;
    border-radius: 50%;
    width: 100px;
    height: 100px; }
  img.shaft-profile-avatar-150 {
    position: relative;
    -webkit-border-radius: 50%;
    -moz-border-radius: 50%;
    border-radius: 50%;
    width: 150px;
    height: 150px; }
  img.shaft-profile-avatar-200 {
    position: relative;
    -webkit-border-radius: 50%;
    -moz-border-radius: 50%;
    border-radius: 50%;
    width: 200px;
    height: 200px; }
  img.shaft-profile-avatar-250 {
    position: relative;
    -webkit-border-radius: 50%;
    -moz-border-radius: 50%;
    border-radius: 50%;
    width: 250px;
    height: 250px; }

.shaft-share-buttons {
  font-size: 20px; }
  .shaft-share-buttons a {
    text-decoration: none !important; }

.navbar-toggle {
  border: none;
  background: transparent !important; }
  .navbar-toggle:hover {
    background: transparent !important; }
  .navbar-toggle .icon-bar {
    width: 22px;
    transition: all 0.2s; }
  .navbar-toggle .top-bar {
    transform: rotate(45deg);
    transform-origin: 10% 10%; }
  .navbar-toggle .middle-bar {
    opacity: 0; }
  .navbar-toggle .bottom-bar {
    transform: rotate(-45deg);
    transform-origin: 10% 90%; }
  .navbar-toggle.collapsed .top-bar {
    transform: rotate(0); }
  .navbar-toggle.collapsed .middle-bar {
    opacity: 1; }
  .navbar-toggle.collapsed .bottom-bar {
    transform: rotate(0); }

textarea.form-control {
  height: auto;
  min-height: 100px; }

/*# sourceMappingURL=shaft.css.map */
