

{% call widget.panel(_("Edit Account Info")) %}
    {% call forms.post(request.endpoint) %}

        {{ forms.input(label="Nickname", name="name", value=current_user.name, data={"validation": "required"}) }}

        {{ forms.input(label="First Name", name="first_name", value=current_user.first_name or '') }}

        {{ forms.input(label="Last Name", name="last_name", value=current_user.last_name or '') }}

        {{ forms.date_picker(label='Date of Birth', name='dob', value=current_user.dob or '') }}

        {{ forms.input(label="Address", name="address", value=current_user.address or '') }}

        {{ forms.input(label="City", name="address_city", value=current_user.address_city or '') }}

        {{ forms.input(label="State/Province", name="address_state", value=current_user.address_state or '') }}

        {{ forms.input(label="Zip/Postal Code", name="address_zip_code", value=current_user.address_zip_code or '') }}

        {{ forms.input(label="Country", name="address_country", value=current_user.address_country or '') }}

        {{ forms.input(label="Telephone", name="telephone", value=current_user.telephone or '') }}

        {{ forms.button('Save Account Info', type='submit', class='btn-info') }}
    {% endcall %}
{% endcall %}

