


{% call widget.panel( login_title or 'Lost Password') %}

    {% call forms.post(request.endpoint) %}

        {{ forms.hidden("next", value=login_url_next) }}

        {{ forms.input(label="Email", name="email", type="email", data={"validation": "email"}) }}

        {{ forms.button("Retrieve Password", type="submit", class="btn btn-primary btn-lg btn-block") }}

        {% call forms.group() %}
            {{ recaptcha }}
        {% endcall %}

    {% endcall %}

    <hr>

    <a href="{{url_for("AuthLogin:login")}}"><i class="fa fa-arrow-left"></i> Login</a>

{% endcall %}

