
{% call widget.panel(_("Change Pofile Image")) %}
    <div>
        {% if current_user.profile_image_url %}
            <div>
                <img src="{{ current_user.profile_image_url }}" class="responsive">
            </div>
        {% endif %}
    </div>
    {% call forms.upload(request.endpoint) %}
        {{ forms.input(label="Select Image to upload", type='file',
            name='file') }}
        {{ forms.button('Upload Profile Picture', type='submit', class='btn-info') }}
    {% endcall %}
{% endcall %}